@@ -8,14 +8,14 @@ Z1 = bivariate_normal(X, Y, 1.0, 1.0, 0.0, 0.0)
 Z2 = bivariate_normal(X, Y, 1.5, 0.5, 1, 1)
 
 # difference of Gaussians
-#im = imshow(Z2-Z1, interpolation='bilinear', origin='lower')
-axis('off')
+#im = imshow(Z2-Z1, interpolation='bilinear', origin='lower', cmap=cm.bone)
+#axis('off')
+# see contour_demo2.py for a colorbar example
 levels, colls = contour(Z2-Z1, levels=6,
                         linewidths=arange(.5, 4, .5),
                         colors=('r', 'green', 'blue', (1,1,0), '#afeeee', 0.5),
                         origin='lower')
 legend(loc='lower left')
-#colorbar()
 savefig('test')
 show()
 
