@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-from matplotlib.matlab import *
+from pylab import *
 
 delta = 0.025
 x = y = arange(-3.0, 3.0, delta)
@@ -8,8 +8,7 @@ Z1 = bivariate_normal(X, Y, 1.0, 1.0, 0.0, 0.0)
 Z2 = bivariate_normal(X, Y, 1.5, 0.5, 1, 1)
 
 # difference of Gaussians
-im = imshow(Z2-Z1, interpolation='bilinear')
+im = imshow(Z2-Z1, interpolation='bilinear', origin='lower')
 axis('off')
-#savefig('image_demo')
 show()
 
