@@ -561,9 +561,13 @@ class LocationEvent(Event):
         self.x = x
         self.y = y
 
+        if self.x is None or self.y is None:
+            # cannot check if event was in axes if no x,y info
+            return
+        
         self.inaxes = None
         for a in self.canvas.figure.get_axes():
-            if a.in_axes(self.x, self.y):
+            if self.x is not None and self.y is not None and a.in_axes(self.x, self.y):
                 self.inaxes = a
                 xdata, ydata = a.transData.inverse_xy_tup((self.x, self.y))
                 self.xdata  = xdata
@@ -665,7 +669,7 @@ class FigureCanvasBase:
 
         self._button     = None  # the button pressed
         self._key        = None  # the key pressed
-        self._lastx, self.last_y = None, None
+        self._lastx, self._lasty = None, None
 
     def key_press_event(self, key):
         self._key = key
