@@ -48,7 +48,8 @@ DEBUG = False
 PIXELS_PER_INCH = 96
 
 # Image formats that this backend supports - for FileChooser and print_figure()
-IMAGE_FORMAT          = ['eps', 'jpg', 'png', 'ps', 'svg'] + ['bmp'] # , 'raw', 'rgb']
+IMAGE_FORMAT          = ['bmp', 'eps', 'jpg', 'png', 'ps', 'svg']
+#IMAGE_FORMAT          = ['bmp', 'eps', 'jpg', 'png', 'pdf', 'ps', 'svg'] # pdf not ready yet
 IMAGE_FORMAT.sort()
 IMAGE_FORMAT_DEFAULT  = 'png'
 
@@ -320,7 +321,18 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
                 from backend_agg import FigureCanvasAgg  as FigureCanvas
             except:
                 error_msg('Save figure failure:\n'
-                          'Agg must be loaded to save as bmp, raw and rgb',
+                          'Agg must be installed to save as bmp, raw and rgb',
+                          parent=self)                
+            else:
+                fc = self.switch_backends(FigureCanvas)
+                fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
+
+        elif ext in ('pdf',):
+            try: 
+                from backend_cairo import FigureCanvasCairo  as FigureCanvas
+            except:
+                error_msg('Save figure failure:\n'
+                          'Cairo must be installed to save as pdf',
                           parent=self)                
             else:
                 fc = self.switch_backends(FigureCanvas)
@@ -345,6 +357,7 @@ class FigureManagerGTK(FigureManagerBase):
     canvas      : The FigureCanvas instance
     num         : The Figure number
     toolbar     : The gtk.Toolbar  (gtk only)
+    vbox        : The gtk.VBox containing the canvas and toolbar (gtk only)
     window      : The gtk.Window   (gtk only)
     """
     def __init__(self, canvas, num):
@@ -354,12 +367,12 @@ class FigureManagerGTK(FigureManagerBase):
         self.window = gtk.Window()
         self.window.set_title("Figure %d" % num)
 
-        vbox = gtk.VBox()
-        self.window.add(vbox)
-        vbox.show()
+        self.vbox = gtk.VBox()
+        self.window.add(self.vbox)
+        self.vbox.show()
 
         self.canvas.show()
-        vbox.pack_start(self.canvas, True, True)
+        self.vbox.pack_start(self.canvas, True, True)
 
         # must be inited after the window, drawingArea and figure
         # attrs are set
@@ -372,7 +385,7 @@ class FigureManagerGTK(FigureManagerBase):
 
         if self.toolbar != None:
             self.toolbar.show()
-            vbox.pack_end(self.toolbar, False, False)
+            self.vbox.pack_end(self.toolbar, False, False)
 
         def destroy(*args): Gcf.destroy(num)
         self.window.connect("destroy", destroy)
