@@ -106,7 +106,7 @@ ACCEPTS: float
         self._imcache = None
         cm.ScalarMappable.changed(self)
 
-    def make_image(self, flipy):        
+    def make_image(self, flipy):
         if self._A is not None:
             if self._imcache is None:
                 x = self.to_rgba(self._A, self._alpha)
@@ -117,6 +117,7 @@ ACCEPTS: float
         else:
             raise RuntimeError('You must first set the image array or the image attribute')
 
+
         bg = colorConverter.to_rgba(self.axes.get_frame().get_facecolor(), 0)
         im.set_bg( *bg)
         im.is_grayscale = (self.cmap.name == "gray" and
@@ -138,7 +139,7 @@ ACCEPTS: float
         # the viewport scale factor
         sx = dxintv/self.axes.viewLim.width()
         sy = dyintv/self.axes.viewLim.height()
-
+            
         if im.get_interpolation()!=_image.NEAREST:
             im.apply_translation(-1, -1)
 
@@ -151,6 +152,7 @@ ACCEPTS: float
 
         l, b, widthDisplay, heightDisplay = self.axes.bbox.get_bounds()
 
+
         im.apply_translation(tx, ty)
         im.apply_scaling(sx, sy)
 
@@ -158,10 +160,12 @@ ACCEPTS: float
         rx = widthDisplay / numcols
         ry = heightDisplay  / numrows
 
+        
         if im.get_aspect()==_image.ASPECT_PRESERVE:
             if ry < rx: rx = ry
             # todo: center the image in viewport
             im.apply_scaling(rx, rx)
+            
         else:
             im.apply_scaling(rx, ry)
 
@@ -170,6 +174,7 @@ ACCEPTS: float
         return im
     
     def draw(self, renderer, *args, **kwargs):
+
         if not self.get_visible(): return 
         isUpper = self.origin=='upper'
         flipy = renderer.flipy()
@@ -258,21 +263,22 @@ ACCEPTS: ['bicubic' | 'bilinear' | 'blackman100' | 'blackman256' | 'blackman64',
         if not self._interpd.has_key(s):
             raise ValueError('Illegal interpolation string')
         self._interpolation = s
-
-
-
-
         
     def get_extent(self):
         'get the image extent: xmin, xmax, ymin, ymax'
         if self._extent is not None:
             return self._extent
-        else:
+        else:            
             numrows, numcols = self.get_size()
-            width, height = numcols, numrows
-            return 0, width, 0, height
-
+            iwidth, iheight = numcols, numrows
+            #return 0, width, 0, height
+            tmp, tmp, dwidth, dheight = self.axes.bbox.get_bounds()
+            sx = dwidth  / iwidth
+            sy = dheight / iheight
 
+            if self.get_aspect()=='preserve' and sy<sx: sx = sy 
+            return 0, 1.0/sx*dwidth, 0, 1.0/sy*dheight
+        
 class FigureImage(Artist, cm.ScalarMappable):
     def __init__(self, fig,
                  cmap = None,
