@@ -263,7 +263,7 @@ FT2Font::clear(const Py::Tuple & args) {
   image.height  = 0;
   image.offsetx = 0;
   image.offsety = 0;
-  
+
   text  = "";
   angle = 0.0;
   
@@ -277,12 +277,16 @@ FT2Font::clear(const Py::Tuple & args) {
   for (size_t i=0; i<gms.size(); i++) {
     Py_DECREF(gms[i]);
   }
+
   glyphs.resize(0);
   gms.resize(0);
 
   return Py::Object();
 }
 
+
+
+
 char FT2Font::set_size__doc__[] = 
 "set_size(ptsize, dpi)\n"
 "\n"
@@ -305,6 +309,27 @@ FT2Font::set_size(const Py::Tuple & args) {
   return Py::Object();
 }
 
+
+char FT2Font::set_charmap__doc__[] = 
+"set_charmap(i)\n"
+"\n"
+"Make the i-th charmap current\n"
+;
+
+Py::Object
+FT2Font::set_charmap(const Py::Tuple & args) {
+  _VERBOSE("FT2Font::set_charmap");
+  args.verify_length(1);
+  
+  int i = Py::Int(args[0]);
+  if (i>=face->num_charmaps) 
+    throw Py::ValueError("i exceeds the available number of char maps");
+  FT_CharMap charmap = face->charmaps[i];
+  if (FT_Set_Charmap( face, charmap ))
+    throw Py::ValueError("Could not set the charmap");
+  return Py::Object();
+}
+
 FT_BBox
 FT2Font::compute_string_bbox( ) {
   _VERBOSE("FT2Font::compute_string_bbox");
@@ -321,10 +346,11 @@ FT2Font::compute_string_bbox( ) {
     if ( glyph_bbox.yMin < bbox.yMin ) bbox.yMin = glyph_bbox.yMin;
     if ( glyph_bbox.xMax > bbox.xMax ) bbox.xMax = glyph_bbox.xMax;
     if ( glyph_bbox.yMax > bbox.yMax ) bbox.yMax = glyph_bbox.yMax;
-  } /* check that we really grew the string bbox */
+  }
+  /* check that we really grew the string bbox */
   if ( bbox.xMin > bbox.xMax ) {
     bbox.xMin = 0; bbox.yMin = 0; bbox.xMax = 0; bbox.yMax = 0;
-  } /* return string bbox */
+  }
   return bbox;
 }
 
@@ -347,7 +373,7 @@ FT2Font::load_glyphs() {
     if ( use_kerning && previous && glyph_index ) { 
       FT_Vector delta;
       FT_Get_Kerning( face, previous, glyph_index,
-		      ft_kerning_default, &delta );
+		      FT_KERNING_DEFAULT, &delta );
       pen.x += delta.x;
     }
     error = FT_Load_Glyph( face, glyph_index, FT_LOAD_DEFAULT ); 
@@ -390,7 +416,7 @@ FT2Font::set_text(const Py::Tuple & args) {
   _VERBOSE("FT2Font::set_text");
   args.verify_length(2);
   text = Py::String(args[0]);
-  double angle = Py::Float(args[1]);
+  angle = Py::Float(args[1]);
   
   angle = angle/360.0*2*3.14159;
   //this computes width and height in subpixels so we have to divide by 64
@@ -462,13 +488,13 @@ FT2Font::load_char(const Py::Tuple & args) {
   int error = FT_Load_Char( face, (unsigned long)charcode, FT_LOAD_DEFAULT);
   
   if (error)
-    throw Py::RuntimeError("Could not load charcode");
+    throw Py::RuntimeError(Printf("Could not load charcode %d", charcode).str());
   
   FT_Glyph thisGlyph;
   error = FT_Get_Glyph( face->glyph, &thisGlyph );
   
   if (error)
-    throw Py::RuntimeError("Could not get glyph for char");
+    throw Py::RuntimeError(Printf("Could not get glyph for char %d", charcode).str());
   
   size_t num = glyphs.size();  //the index into the glyphs list
   glyphs.push_back(thisGlyph);
@@ -490,7 +516,7 @@ FT2Font::get_width_height(const Py::Tuple & args) {
   _VERBOSE("FT2Font::get_width_height");
   args.verify_length(0);
   
-  FT_BBox bbox =  compute_string_bbox();
+  FT_BBox bbox = compute_string_bbox();
   
   Py::Tuple ret(2);
   ret[0] = Py::Int(bbox.xMax - bbox.xMin);
@@ -498,6 +524,22 @@ FT2Font::get_width_height(const Py::Tuple & args) {
   return ret;
 }
 
+char FT2Font::get_descent__doc__[] = 
+"d = get_descent()\n"
+"\n"
+"Get the descent of the current string set by set_text in 26.6 subpixels.\n"
+"The rotation of the string is accounted for.  To get the descent\n"
+"in pixels, divide this value by 64.\n"
+;
+Py::Object
+FT2Font::get_descent(const Py::Tuple & args) {
+  _VERBOSE("FT2Font::get_descent");
+  args.verify_length(0);
+  
+  FT_BBox bbox = compute_string_bbox();
+  return Py::Int(- bbox.yMin);;
+}
+
 void
 FT2Font::draw_bitmap( FT_Bitmap*  bitmap,
 		      FT_Int      x,
@@ -516,6 +558,7 @@ FT2Font::draw_bitmap( FT_Bitmap*  bitmap,
 	  if ( i >= width || j >= height )
 	    continue;
 	  image.buffer[i + j*width] |= bitmap->buffer[q*bitmap->width + p];
+
 	}
     }
 }
@@ -572,7 +615,7 @@ FT2Font::draw_rect(const Py::Tuple & args) {
   if ( x0<0 || y0<0 || x1<0 || y1<0 || 
        x0>iwidth || x1>iwidth ||
        y0>iheight || y1>iheight ) 
-    throw Py::ValueError("rect coords outside image bounds");
+    throw Py::ValueError("Rect coords outside image bounds");
   
   for (long i=x0; i<x1; ++i) {
     image.buffer[i + y0*iwidth] = 255;
@@ -614,9 +657,10 @@ char FT2Font::draw_glyphs_to_bitmap__doc__[] =
 ;
 Py::Object
 FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
+
   _VERBOSE("FT2Font::draw_glyphs_to_bitmap");
   args.verify_length(0);
-  
+
   FT_BBox string_bbox = compute_string_bbox();
   
   image.width   = (string_bbox.xMax-string_bbox.xMin) / 64+2;
@@ -625,19 +669,21 @@ FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
   image.offsetx = (int)(string_bbox.xMin/64.0);
   if (angle==0)  
     image.offsety = -image.height;
-  else
+  else {
     image.offsety = (int)(-string_bbox.yMax/64.0);
-  
+  }
+
   size_t numBytes = image.width*image.height;
   delete [] image.buffer;
   image.buffer = new unsigned char [numBytes];
   for (size_t n=0; n<numBytes; n++) 
     image.buffer[n] = 0;
+
+
   
   for ( size_t n = 0; n < glyphs.size(); n++ )
     {
       FT_BBox bbox;
-      
       FT_Glyph_Get_CBox(glyphs[n], ft_glyph_bbox_pixels, &bbox);
       
       error = FT_Glyph_To_Bitmap(&glyphs[n],
@@ -648,17 +694,20 @@ FT2Font::draw_glyphs_to_bitmap(const Py::Tuple & args) {
 				 );
       if (error)
 	throw Py::RuntimeError("Could not convert glyph to bitmap");
-      
+
       FT_BitmapGlyph bitmap = (FT_BitmapGlyph)glyphs[n];
-      /* now, draw to our target surface (convert position) */
+      // now, draw to our target surface (convert position)
       
       //bitmap left and top in pixel, string bbox in subpixel
-      
-      draw_bitmap( &bitmap->bitmap, 
-		   bitmap->left-string_bbox.xMin/64,
-		   string_bbox.yMax/64-bitmap->top+1
-		   );
+      FT_Int x = (FT_Int)(bitmap->left-string_bbox.xMin/64.);
+      FT_Int y = (FT_Int)(string_bbox.yMax/64.-bitmap->top+1);
+      //make sure the index is non-neg
+      x = x<0?0:x;
+      y = y<0?0:y;
+
+      draw_bitmap( &bitmap->bitmap, x, y);
     }
+
   return Py::Object();
 }
 
@@ -725,7 +774,7 @@ FT2Font::get_glyph_name(const Py::Tuple & args) {
 
   char buffer[128];
   if (FT_Get_Glyph_Name(face, (FT_UInt) Py::Int(args[0]), buffer, 128))
-    Py::RuntimeError("Could not get glyph names.");
+    throw Py::RuntimeError("Could not get glyph names.");
   return Py::String(buffer);
 }
 
@@ -744,8 +793,7 @@ FT2Font::get_charmap(const Py::Tuple & args) {
   Py::Dict charmap;
   
   FT_ULong code = FT_Get_First_Char(face, &index);
-  while (code != 0) {
-    //charmap[Py::Long((long) code)] = Py::Int((int) index);
+  while (index != 0) {
     charmap[Py::Int((int) index)] = Py::Long((long) code);
     code = FT_Get_Next_Char(face, code, &index);
   }
@@ -1146,8 +1194,13 @@ FT2Font::init_type() {
 		     FT2Font::set_text__doc__);
   add_varargs_method("set_size", &FT2Font::set_size,
 		     FT2Font::set_size__doc__);
+  add_varargs_method("set_charmap", &FT2Font::set_charmap,
+		     FT2Font::set_charmap__doc__);
+
   add_varargs_method("get_width_height", &FT2Font::get_width_height,
 		     FT2Font::get_width_height__doc__);
+  add_varargs_method("get_descent", &FT2Font::get_descent,
+		     FT2Font::get_descent__doc__);
   add_varargs_method("get_glyph_name", &FT2Font::get_glyph_name,
 		     FT2Font::get_glyph_name__doc__);
   add_varargs_method("get_charmap", &FT2Font::get_charmap,
