@@ -18,7 +18,7 @@ extern "C" {
 #include "CXX/Extensions.hxx"
 
 
-// the freetype string renderered into a width, height buffer
+// the freetype string rendered into a width, height buffer
 class FT2Image {
 public:
   FT2Image();
@@ -52,11 +52,13 @@ public:
   Py::Object set_bitmap_size(const Py::Tuple & args);
   Py::Object clear(const Py::Tuple & args);
   Py::Object set_size(const Py::Tuple & args);
+  Py::Object set_charmap(const Py::Tuple & args);
   Py::Object set_text(const Py::Tuple & args);
   Py::Object get_glyph(const Py::Tuple & args);
   Py::Object get_num_glyphs(const Py::Tuple & args);
   Py::Object load_char(const Py::Tuple & args);
   Py::Object get_width_height(const Py::Tuple & args);
+  Py::Object get_descent(const Py::Tuple & args);
   Py::Object write_bitmap(const Py::Tuple & args);
   Py::Object draw_rect(const Py::Tuple & args);
   Py::Object image_as_str(const Py::Tuple & args);
@@ -90,15 +92,18 @@ private:
   FT_BBox compute_string_bbox();
   void load_glyphs();
   void draw_bitmap( FT_Bitmap*  bitmap, FT_Int x, FT_Int y);
+  void set_scalable_attributes();
 
   static char set_bitmap_size__doc__ [];
   static char clear__doc__ [];
   static char set_size__doc__ [];
+  static char set_charmap__doc__ [];
   static char set_text__doc__ [];
   static char get_glyph__doc__ [];
   static char get_num_glyphs__doc__ [];
   static char load_char__doc__ [];
   static char get_width_height__doc__ [];
+  static char get_descent__doc__ [];
   static char write_bitmap__doc__ [];
   static char draw_rect__doc__ [];
   static char image_as_str__doc__ [];
